# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import json
from django.shortcuts import get_object_or_404
from django.utils.dateformat import format

from common.utils import JSONResponse
from ci.models import CiProject, JenkinsBuild

logger = logging.getLogger('qa_dashboard')


def project(request, name):
    project = get_object_or_404(
        CiProject,
        launchpad_project__name=name,
        release__name='head',
    )

    coverage_builds = JenkinsBuild.objects.filter(
        jenkins_job=project.autolanding_primary_job,
        coverage_report__isnull=False,
    ).order_by('timestamp')

    data = []
    line_data = {
        'key': 'Line coverage',
        'color': '#DD4814'
    }
    branch_data = {
        'key': 'Branch coverage',
        'color': '#772953'
    }

    line_values = []
    branch_values = []

    for build in coverage_builds:
        ld = dict(
            y=build.line_coverage,
            x=format(build.timestamp, 'U'),
        )
        bd = dict(
            y=build.branch_coverage,
            x=format(build.timestamp, 'U'),
        )
        line_values.append(ld)
        branch_values.append(bd)
    line_data['values'] = line_values
    branch_data['values'] = branch_values

    data.append(line_data)
    data.append(branch_data)

    data = json.dumps(data)

    return JSONResponse(data)
