# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock, patch
import datetime
import json
import os.path
import unittest

from ci.util.extractor import JenkinsBuild
import utilities

"""Tests for class JenkinsBuild"""

DATA_PATH = "ci/tests/data"
BUILD_FILE = "nux-mbs-autolanding.jenkins.build"


class JenkinsBuildTestCase(unittest.TestCase):
    """Unit tests for class JenkinsBuild"""

    def setUp(self):
        url = "http://10.97.0.6:8080/view/mbs/job/nux-mbs-autolanding/98/"
        data = utilities.get_json_data(DATA_PATH,
                                       "nux-mbs-autolanding-98.json")
        self.build = JenkinsBuild(url, data)
        self.run_list = ['nux-mbs-autolanding-98-amd64.json',
                         'nux-mbs-autolanding-98-armhf.json',
                         'nux-mbs-autolanding-98-i386.json']
        self.run_data = utilities.get_json_data_list(DATA_PATH, self.run_list)
        self.report_data = utilities.get_json_data(DATA_PATH,
                                                   "nux-mbs-autolanding-98-"
                                                   "amd64-testReport.json")

    def test_str(self):
        self.assertIn("<JenkinsBuild", "%s" % (self.build))
        self.assertIn("nux-mbs-autolanding", "%s" % (self.build))

    def test_name(self):
        self.assertEqual(self.build.name, "nux-mbs-autolanding #98")

    def test_number(self):
        self.assertEqual(self.build.number, 98)

    def test_duration(self):
        self.assertEqual(self.build.duration.seconds,
                         datetime.timedelta(seconds=11, minutes=77).seconds)

    def test_timestamp(self):
        self.assertEqual(self.build.timestamp,
                         datetime.datetime(2013, 1, 11, 10, 52, 52))

    def test_runs(self):
        runs = self.build.runs
        self.assertEqual(len(runs), 3)

    def test_flavors_none(self):
        self.assertEqual(self.build.flavor, None)

    def test_series_none(self):
        self.assertEqual(self.build.series, None)

    def test_result(self):
        self.assertEqual(self.build.result, "SUCCESS")

    def test_result_null(self):
        url = "http://10.97.0.6:8080/view/mbs/job/nux-mbs-autolanding/98/"
        data = utilities.get_json_data(DATA_PATH,
                                       "compiz-mbs-autolanding-193.json")
        build = JenkinsBuild(url, data)
        self.assertEqual(build.result, None)

    def test_building(self):
        self.assertEqual(self.build.building, False)

    def test_building_true(self):
        url = "http://10.97.0.6:8080/view/mbs/job/nux-mbs-autolanding/98/"
        data = utilities.get_json_data(DATA_PATH,
                                       "compiz-mbs-autolanding-193.json")
        build = JenkinsBuild(url, data)
        self.assertEqual(build.building, True)

    def test_parameters(self):
        self.assertEqual(len(self.build.parameters), 19)

    def test_console_url(self):
        self.assertEqual(self.build.get_console_url(),
                         'http://10.97.0.6:8080/view/mbs/job/'
                         'nux-mbs-autolanding/98/consoleText')

    def test_test_report_url(self):
        self.assertEqual(self.build.get_test_report_url(),
                         'http://10.97.0.6:8080/view/mbs/job/'
                         'nux-mbs-autolanding/98/testReport')

    def test_coverage_report_url(self):
        self.assertEqual(self.build.get_coverage_report_url(),
                         'http://10.97.0.6:8080/view/mbs/job/'
                         'nux-mbs-autolanding/98/'
                         'artifact/work/results/coverage.xml')
