# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock, patch
import datetime
import json
import os.path
import unittest

from ci.util.extractor import (JenkinsJob, JenkinsBuild)
import utilities

"""Tests for class JenkinsJob"""

DATA_PATH = "ci/tests/data"


class JenkinsJobTestCase(unittest.TestCase):
    """Unit tests for class JenkinsJob"""

    def setUp(self):
        url = "http://10.97.0.6:8080/job/nux-mbs-autolanding/"
        data = self._get_data("nux-mbs-autolanding.json")
        self.job = JenkinsJob(url, data)

    def _get_data(self, filename):
        return utilities.get_json_data(DATA_PATH, filename)

    def test_str(self):
        self.assertIn("<JenkinsJob", "%s" % (self.job))
        self.assertIn("nux-mbs-autolanding", "%s" % (self.job))

    def test_name(self):
        self.assertEqual(self.job.name, "nux-mbs-autolanding")

    def test_builds(self):
        builds = self.job.builds
        self.assertEqual(len(builds), 101)

    def test_downstream_jobs(self):
        jobs = self.job.downstream_jobs
        name_list = [j.name for j in jobs]
        self.assertEqual(len(jobs), 3)
        self.assertIn("generic-land", name_list)

    def test_buildable(self):
        self.assertEqual(self.job.buildable, True)

    def test_in_queue(self):
        self.assertEqual(self.job.in_queue, False)

    def test_last_build(self):
        self.assertEqual(self.job.last_build, 101)

    def test_last_completed_build(self):
        self.assertEqual(self.job.last_completed_build, 101)

    def test_last_failed_build(self):
        self.assertEqual(self.job.last_failed_build, 101)

    def test_last_successful_build(self):
        self.assertEqual(self.job.last_successful_build, 99)

    def test_get_build_url_from_number(self):
        self.assertEqual(self.job.get_build_url_from_number(21),
                         'http://10.97.0.6:8080/job/nux-mbs-autolanding/21')
