# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock, patch
import datetime
import json
import os.path
import unittest

from ci.util.extractor import JenkinsTestReport
import utilities

"""Tests for class JenkinsTestReport"""

DATA_PATH = "ci/tests/data"
DATA_FILE = "nux-mbs-autolanding-98-amd64-testReport.json"


class JenkinsTestReportTestCase(unittest.TestCase):

    """Unit tests for class JenkinsTestReport"""

    def setUp(self):
        url = ("http://10.97.0.6:8080/job/nux-mbs-autolanding/"
               "build=pbuilder,distribution=raring,flavor=amd64/98/testReport")
        data = utilities.get_json_data(DATA_PATH, DATA_FILE)
        self.test_report = JenkinsTestReport(url, data)

    def test_data_none(self):
        test_report = None
        url = ("http://10.97.0.6:8080/job/nux-mbs-autolanding/"
               "build=pbuilder,distribution=raring,flavor=amd64/98/testReport")
        data = utilities.get_json_data(DATA_PATH, DATA_FILE)
        with patch('ci.util.extractor.get_json_from_url',
                   MagicMock(return_value=data)):
            test_report = JenkinsTestReport(url)
        self.assertEqual(self.test_report.pass_count, 131)

    def test_fail_count(self):
        self.assertEqual(self.test_report.fail_count, 0)

    def test_pass_count(self):
        self.assertEqual(self.test_report.pass_count, 131)

    def test_skip_count(self):
        self.assertEqual(self.test_report.skip_count, 0)

    def test_duration(self):
        self.assertEqual(self.test_report.duration.microseconds, 0)

    def test_suites(self):
        self.assertEqual(len(self.test_report.suites), 21)

    def test_cases(self):
        self.assertEqual(len(self.test_report.suites[0].cases), 8)
