# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from urlparse import urljoin
from mock import patch, MagicMock
from django.test import TestCase
from unittest import skip
from nose.plugins.attrib import attr
from ci.models import (JenkinsBuild,
                         JenkinsJob,
                         JenkinsCoverageReport,
                         JenkinsTestReport)
from ci.util.jenkins_job import create_jenkins_job_from_url
from ci.util.jenkins_build import create_jenkins_build_from_url


# obviously these need to updated if something changes
JENKINS_BASE_URL = "http://10.97.2.10:8080"
JENKINS_JOB_URL = 'job/phone-app-raring-i386-autolanding'
JENKINS_BUILD_URL = 'job/phone-app-raring-i386-autolanding/26'


class JenkinsBuildTestCase(TestCase):

    def setUp(self):
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()
        # slurp a vanilla JenkinsJob and JenkinsBuild
        self.jenkins_job_url = urljoin(
            JENKINS_BASE_URL,
            JENKINS_JOB_URL)
        self.jenkins_job = create_jenkins_job_from_url(
            self.jenkins_job_url,
            create_builds=False,
            create_downstream_jobs=False)
        self.jenkins_build_url = urljoin(
            JENKINS_BASE_URL,
            JENKINS_BUILD_URL)
        self.jenkins_build = create_jenkins_build_from_url(
            self.jenkins_build_url,
            self.jenkins_job)

    def tearDown(self):
        self.logger_patch.stop()


class TestBuildCreation(JenkinsBuildTestCase):

    @attr('qa_lab')
    def test_we_got_the_build(self):
        self.assertIsNotNone(JenkinsBuild.objects.get(
                url=self.jenkins_build_url))


    @attr('qa_lab')
    def test_coverage_report_exists(self):
        coverage_report = JenkinsCoverageReport.objects.get(
            jenkins_build=self.jenkins_build)
        self.assertIsNotNone(coverage_report)


    @attr('qa_lab')
    def test_coverage_report_line_rate_is_correct(self):
        coverage_report = JenkinsCoverageReport.objects.get(
            jenkins_build=self.jenkins_build)
        self.assertIsNotNone(coverage_report)
        self.assertAlmostEqual(0.45,
                               coverage_report.line_rate,
                               places=2)
        

    @attr('qa_lab')
    def test_coverage_report_branch_rate_is_correct(self):
        coverage_report = JenkinsCoverageReport.objects.get(
            jenkins_build=self.jenkins_build)
        self.assertIsNotNone(coverage_report)
        self.assertAlmostEqual(0.53,
                               coverage_report.branch_rate,
                               places=2)


    @attr('qa_lab')
    def test_test_report_exists(self):
        self.assertIsNotNone(self.jenkins_build.test_report)


    @attr('qa_lab')
    def test_test_report_fail_count_is_correct(self):
        test_report = self.jenkins_build.test_report
        self.assertEqual(0, test_report.fail_count)


    @attr('qa_lab')
    def test_test_report_pass_count_is_correct(self):
        test_report = self.jenkins_build.test_report
        self.assertEqual(98, test_report.pass_count)


    @attr('qa_lab')
    def test_test_report_skip_count_is_correct(self):
        test_report = self.jenkins_build.test_report
        self.assertEqual(98, test_report.pass_count)


    @attr('qa_lab')
    def test_test_report_duration_is_correct(self):
        test_report = self.jenkins_build.test_report
        self.assertEqual(0, test_report.duration)



class TestDuplicateJenkinsBuild(JenkinsBuildTestCase):

    @skip
    @attr('qa_lab')
    def test_create_duplicate_does_nothing(self):
        self.assertIsNotNone(
            JenkinsBuild.objects.get(url=self.jenkins_build_url))
        result = self.jenkins_build = create_jenkins_job_from_url(
            self.jenkins_build_url,
            self.jenkins_job)
        self.assertIsNone(result)
        # this would error if more than one were returned
        self.assertIsNotNone(
            JenkinsBuild.objects.get(url=self.jenkins_build_url))
