# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from urlparse import urljoin
from mock import patch, MagicMock
from django.test import TestCase
from unittest import skip
from nose.plugins.attrib import attr
from ci.models import (JenkinsBuild,
                         JenkinsJob,
                         JenkinsCoverageReport,
                         JenkinsTestReport)
from ci.util.jenkins_job import (create_jenkins_job_from_url,
                                       update_jenkins_job)

from ci.util.jenkins_build import create_jenkins_build_from_url


# obviously these need to updated if something changes
JENKINS_BASE_URL = "http://10.97.2.10:8080"
JENKINS_JOB_URL = 'job/phone-app-raring-i386-autolanding'
JENKINS_BUILD_URL = 'job/phone-app-raring-i386-autolanding/26'



class TestJenkinsJobCreateFromUrl(TestCase):

    def setUp(self):
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()

    def tearDown(self):
        self.logger_patch.stop()

    @attr('qa_lab')
    def test_no_builds_param_doesnt_create_builds(self):
        self.jenkins_job_url = urljoin(
            JENKINS_BASE_URL,
            JENKINS_JOB_URL)
        self.jenkins_job = create_jenkins_job_from_url(
            self.jenkins_job_url,
            create_builds=False)
        self.assertIsNotNone(
            JenkinsJob.objects.get(url=self.jenkins_job_url))
        self.assertEqual(
            0,
            len(JenkinsBuild.objects.all()))


class TestDuplicateJenkinsJob(TestCase):
    fixtures = ['duplicate_job_fixture.json']

    def setUp(self):
        self.logger_patch = patch('ci.models.logger')
        #self.logger_patch.start()
        # slurp a vanilla JenkinsJob and JenkinsBuild
        self.jenkins_job_url = urljoin(
            JENKINS_BASE_URL,
            'job/camera-app-ci/')

    def tearDown(self):
        pass
        #self.logger_patch.stop()

    @skip
    @attr('qa_lab')
    def test_create_duplicate_does_nothing(self):
        self.assertIsNotNone(
            JenkinsJob.objects.get(url=self.jenkins_job_url))
        duplicate_job = create_jenkins_job_from_url(
            self.jenkins_job_url,
            create_builds=False,
            create_downstream_jobs=False)
        # this would error if more than one were returned
        self.assertIsNotNone(
            JenkinsJob.objects.get(url=self.jenkins_job_url))
        self.fail()


class TestUpdate(TestCase):
    fixtures = ['head-indicators.json']

    def setUp(self):
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()

    def tearDown(self):
        self.logger_patch.stop()

    @attr('qa_lab')
    def test_update_job(self):
        jenkins_job = JenkinsJob.objects.get(name='libusermetrics-ci')
        jenkins_build = JenkinsBuild.objects.filter(jenkins_job=jenkins_job).order_by('-timestamp')[0]
        self.assertEqual(7, jenkins_build.number)
        result = update_jenkins_job(jenkins_job)
        jenkins_updated_build = JenkinsBuild.objects.filter(jenkins_job=jenkins_job).order_by('-timestamp')[0]
        self.assertTrue(jenkins_updated_build.number>jenkins_build.number)
