# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
from unittest import skip
from mock import MagicMock, patch
from django.test import TestCase
from ci.models import (CiProject,
                             JenkinsBuild)


class TestUnicode(TestCase):

    def test_vanilla(self):
        ci_project = CiProject.objects.create(
            name='indicator-brass-knuckles')
        self.assertEqual('indicator-brass-knuckles',
                         str(ci_project))


class CiProjectTestCase(TestCase):
    fixtures = ['phablet-apps.json']

    def setUp(self):
        self.phone_app = CiProject.objects.get(
            name="phablet.apps.phone-app")


class TestLastBuild(CiProjectTestCase):

    def test_last_build_is_correct(self):
        last_build = JenkinsBuild.objects.get(
            jenkins_job__name='phone-app-autolanding',
            number=31)
        self.assertEqual(last_build, self.phone_app.last_build)


class TestLastSuccessfulBuild(CiProjectTestCase):

    def test_last_successful_build_is_correct(self):
        last_build = JenkinsBuild.objects.get(
            jenkins_job__name='phone-app-autolanding',
            number=31)
        self.assertEqual(
            last_build,
            self.phone_app.last_successful_build)


#TODO Need to regenerate the fixture to include aggregated coverage data
class TestCoverageReporting(CiProjectTestCase):

    def test_line_coverage_is_correct(self):
        #self.assertAlmostEqual(0.436827711942,
        #                       self.phone_app.line_coverage)
        self.assertIsNone(self.phone_app.line_coverage)

    def test_branch_coverage_is_correct(self):
        #self.assertAlmostEqual(0.528182099721,
        #                       self.phone_app.branch_coverage)
        self.assertIsNone(self.phone_app.branch_coverage)

    @skip
    def test_line_count_is_correct(self):
        #self.assertEqual(??, self.phone_app.line_count)
        #
        self.assertIsNone(self.phone_app.line_count)

    @skip
    def test_hit_count_is_correct(self):
        #self.assertEqual(??, self.phone_app.hit_count)
        #
        self.assertIsNone(self.phone_app.hit_count)

    @skip
    def test_taken_count_is_correct(self):
        #self.assertEqual(??, self.phone_app.taken_count)
        #
        self.assertIsNone(self.phone_app.taken_count)

    @skip
    def test_total_count_is_correct(self):
        #self.assertEqual(??, self.phone_app.total_count)
        #
        self.assertIsNone(self.phone_app.total_count)


#TODO Add some tests with a non-None count
class TestLastFailCount(CiProjectTestCase):

    def test_last_fail_count_is_correct(self):
        self.assertEqual(None, self.phone_app.last_fail_count)


class TestLastPassCount(CiProjectTestCase):

    def test_last_pass_count_is_correct(self):
        self.assertEqual(None, self.phone_app.last_pass_count)


class TestLastSkipCount(CiProjectTestCase):

    def test_last_skip_count_is_correct(self):
        self.assertEqual(None, self.phone_app.last_skip_count)


class TestLastDuration(CiProjectTestCase):

    def test_last_duration_is_correct(self):
        self.assertEqual(None, self.phone_app.last_duration)


class TestShortName(CiProjectTestCase):

    def test_short_name(self):
        self.assertEqual("phone-app", self.phone_app.short_name)


class CiProjectUnitTestCase(TestCase):

    def setUp(self):
        ci_project_mock = MagicMock(
            spec=CiProject,
            return_value=None)
        self.ci_project_patch = patch.object(
            CiProject,
            '__init__',
            new=ci_project_mock)
        self.ci_project_patch.start()
        self.ci_project = CiProject()
        self.logger_patch = patch(
            'ci.models.logger')
        self.logger_patch.start()

    def tearDown(self):
        self.ci_project_patch.stop()
        self.logger_patch.stop()


class TestLastCoverageBuild(CiProjectUnitTestCase):

    def test_last_coverage_build_is_correct(self):
        timestamp = datetime.datetime.now()
        filter_mock = MagicMock(name="foo")
        filter_mock.return_value.__getitem__.return_value = "fake_jenkins_build"
        with patch('ci.models.JenkinsBuild.objects.filter',
                   filter_mock), \
                patch.object(CiProject, 'autolanding_primary_job'):
            result = self.ci_project.last_coverage_build(timestamp)
        self.assertEqual("fake_jenkins_build", result)

    def test_last_coverage_build_doesnt_exist(self):
        timestamp = datetime.datetime.now()
        filter_mock = MagicMock()
        filter_mock.return_value.__getitem__.side_effect = IndexError
        with patch('ci.models.JenkinsBuild.objects.filter',
                   filter_mock), \
                patch.object(CiProject, 'autolanding_primary_job'):
            result = self.ci_project.last_coverage_build(timestamp)
        self.assertIsNone(result)
