# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock, patch, call
from django.test import TestCase
from django.db import IntegrityError
from ci.models import LaunchpadProject, CiProject, Cu2dStack
import ci.util.extractor as extractor
import ci.util.jenkins_job as util_jenkins_job
import ci.util.ci_project as util_ci_project


class CiProjectManagerTestCase(TestCase):

    def setUp(self):
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()

    def tearDown(self):
        self.logger_patch.stop()


class TestCreateFromName(CiProjectManagerTestCase):

    def test_vanilla(self):
        dummy_ci_project_name = 'fake-ci-project'
        launchpad_project_get = MagicMock()
        with patch('ci.models.LaunchpadProject.objects.get',
                   launchpad_project_get), \
                   patch('ci.models.CiProject.objects.create') as ci_project_create:
            result = util_ci_project.create_ci_project_from_name(
                name=dummy_ci_project_name)
        ci_project_create.assert_called_with(
            name=dummy_ci_project_name)
        launchpad_project_get.assert_called_with(
            name=dummy_ci_project_name)
        # TODO come back and assert on ci_project_launchpad_project_add

    def test_launchpad_project_doesnt_exist(self):
        dummy_ci_project_name = 'fake-ci-project'
        ci_project_create = MagicMock()
        launchpad_project_get = MagicMock(
            side_effect=LaunchpadProject.DoesNotExist)
        with patch('ci.models.LaunchpadProject.objects.get',
                   launchpad_project_get), \
               patch('ci.models.CiProject.objects.create',
                     ci_project_create), \
               patch('ci.util.ci_project.create_launchpad_project_from_name'):
            result = util_ci_project.create_ci_project_from_name(
                name=dummy_ci_project_name)
        ci_project_create.assert_called_with(
            name=dummy_ci_project_name)
        launchpad_project_get.assert_called_with(
            name=dummy_ci_project_name)
        # TODO come back and assert on ci_project_launchpad_project_add



class TestCreateFromConfig(CiProjectManagerTestCase):

    def test_vanilla(self):
        with patch('ci.models.Cu2dStack.objects.get'), \
                patch('ci.models.Release.objects.get'), \
                patch('ci.util.ci_project.create_ci_project_from_name') as \
                      create_ci_project_from_name, \
                patch('ci.util.ci_project.create_jenkins_job_from_name') as \
                      jenkins_job_create_from_name, \
                patch('ci.models.CiProject.objects.get') as \
                      ci_project_get:
            create_ci_project_from_name.return_value = MagicMock(
                stack="fake_stack",
                release="fake_release")
            result = util_ci_project.create_ci_project_from_config(
                {'name': 'fake_name',
                 'stack': 'fake_stack',
                 'release': 'fake_release',
                 'ci': 'fake_ci_job_name',
                 'autolanding': 'fake_autolanding_job_name',
                 'server': 'http://s-jenkins:8080'})
        call_args_list = jenkins_job_create_from_name.call_args_list
        self.assertEqual(call('http://s-jenkins:8080', 'fake_ci_job_name'), call_args_list[0])
        self.assertEqual(call('http://s-jenkins:8080', 'fake_autolanding_job_name'),
                         call_args_list[1])
