# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock, patch
from django.test import TestCase
from ci.models import Cu2dStack
import ci.util.cu2d_stack as util_cu2d_stack


class TestCreateFromConfig(TestCase):

    def setUp(self):
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()

    def tearDown(self):
        self.logger_patch.stop()

    def test_vanilla(self):
        series_get_or_create = MagicMock(
            return_value=('fake_series', False))
        release_get_or_create = MagicMock(
            return_value=('fake_release', False))
        cu2d_stack_get_or_create = MagicMock(
            return_value=(MagicMock(), False))
        with patch('ci.models.Series.objects.get_or_create',
                   series_get_or_create), \
                patch('ci.models.Release.objects.get_or_create',
                      release_get_or_create), \
                patch('ci.models.Cu2dStack.objects.get_or_create',
                      cu2d_stack_get_or_create), \
                patch('ci.models.Cu2dStack.save') as cu2d_save, \
                patch('ci.util.cu2d_stack.create_jenkins_job_from_name') as \
                    create_jenkins_job_from_name:
            result = util_cu2d_stack.create_cu2d_stack_from_config(
                {'series': 'fake_series_name',
                 'release': 'fake_release_name',
                 'name': 'fake_stack_name',
                 'ppa': 'fake_ppa',
                 'server': 'fake_server',
                 'job': 'fake_job'})
        create_jenkins_job_from_name.assertCalled()
        self.assertEqual('fake_release', result.release)
        self.assertEqual('fake_series', result.series)
        self.assertEqual('fake_ppa', result.ppa)
