# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock, patch
from django.test import TestCase
from django.db import IntegrityError
from ci.models import JenkinsBuild
import ci.util.extractor as extractor


class TestJenkinsBuild(TestCase):

    def setUp(self):
        jenkins_build_mock = MagicMock(
            spec=JenkinsBuild,
            return_value=None)
        self.jenkins_build_patch = patch.object(
            JenkinsBuild,
            '__init__',
            new=jenkins_build_mock)
        self.jenkins_build_patch.start()
        self.jenkins_build = \
            JenkinsBuild()
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()


    def tearDown(self):
        self.jenkins_build_patch.stop()
        self.logger_patch.stop()


    def test_unicode(self):
        self.jenkins_build.url = 'http://url.of.jenkins/plus/build/42'
        self.assertEqual('http://url.of.jenkins/plus/build/42',
                         str(self.jenkins_build))


class TestCoverageReporting(TestCase):
    fixtures = ['phablet-apps.json']

    def setUp(self):
        self.jenkins_build = JenkinsBuild.objects.get(
            url="http://10.97.2.10:8080/job/phone-app-quantal-i386-autolanding/31/")

    def test_coverage_exists(self):
        self.assertIsNotNone(
            self.jenkins_build.coverage_report)

    def test_line_coverage_is_correct(self):
        self.assertAlmostEqual(0.43703298706,
                               self.jenkins_build.line_coverage)

    def test_branch_coverage_is_correct(self):
        self.assertAlmostEqual(0.526631548142,
                               self.jenkins_build.branch_coverage)



class TestNoCoverageReporting(TestCase):
    """Some builds aren't equipped with coverage, also "primary"
    builds don't manage the coverage results of their downstream
    builds.

    Just return None if no coverage results exist.
    """

    fixtures = ['phablet-apps.json']

    def setUp(self):
        self.jenkins_build = JenkinsBuild.objects.get(
            url="http://10.97.2.10:8080/job/phone-app-autolanding/31/")

    def test_coverage_doesnt_exist(self):
        self.assertIsNone(
            self.jenkins_build.coverage_report)

    def test_line_coverage_is_correct(self):
        self.assertIsNone(self.jenkins_build.line_coverage)

    def test_branch_coverage_is_correct(self):
        self.assertIsNone(self.jenkins_build.branch_coverage)
