# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock, patch
from django.test import TestCase
from lxml.etree import XMLSyntaxError
import ci.util.coverage_report as util_coverage_report


class TestCreateFromCoberturaXml(TestCase):

    def setUp(self):
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()
        self.fake_url = "http://this.is.a.url.com/click/me"
        self.fake_jenkins_build = "fake_jenkins_build"
        self.extractor_jenkins_coverage_params = {
            'lines': 42,
            'hits': 43,
            'branches': 44,
            'taken': 45,
            'total': 46}
        self.extractor_jenkins_coverage_mock = MagicMock(
            **self.extractor_jenkins_coverage_params)
        self.extractor_jenkins_coverage = MagicMock(
            return_value=self.extractor_jenkins_coverage_mock)
        self.extractor_jenkins_coverage_patch = patch(
            'ci.util.extractor.JenkinsCoverage',
            self.extractor_jenkins_coverage)
        self.extractor_jenkins_coverage_patch.start()
        self.jenkins_coverage_report_create_patch = patch(
            'ci.models.JenkinsCoverageReport.objects.create')
        self.jenkins_coverage_report_create = self.jenkins_coverage_report_create_patch.start()

    def tearDown(self):
        self.extractor_jenkins_coverage_patch.stop()
        self.jenkins_coverage_report_create_patch.stop()
        self.logger_patch.stop()

    def test_vanilla(self):
        result = util_coverage_report.create_coverage_report_from_url(
            self.fake_jenkins_build,
            self.fake_url)
        self.jenkins_coverage_report_create.assert_called_with(
            jenkins_build="fake_jenkins_build",
            line_count=42,
            hit_count=43,
            branch_count=44,
            taken_count=45,
            total_count=46)

    def test_xml_syntax_error(self):
        self.extractor_jenkins_coverage.side_effect = XMLSyntaxError
        xml_syntax_error_init_mock = MagicMock(return_value=None)
        with patch('lxml.etree.XMLSyntaxError.__init__', xml_syntax_error_init_mock):
            result = util_coverage_report.create_coverage_report_from_url(
                self.fake_jenkins_build,
                self.fake_url)
        self.assertIsNone(result)
        self.jenkins_coverage_report_create.assert_not_called()
