# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from mock import MagicMock, patch
from django.test import TestCase
from django.db import IntegrityError
from ci.models import JenkinsJob
import ci.util.extractor as extractor


class TestJenkinsJob(TestCase):

    def setUp(self):
        jenkins_job_mock = MagicMock(
            spec=JenkinsJob,
            return_value=None)
        self.jenkins_job_patch = patch.object(
            JenkinsJob,
            '__init__',
            new=jenkins_job_mock)
        self.jenkins_job_patch.start()
        self.jenkins_job = \
            JenkinsJob()
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()

    def tearDown(self):
        self.jenkins_job_patch.stop()
        self.logger_patch.stop()

    def test_unicode(self):
        self.jenkins_job.name = 'deep-thought-autolanding'
        self.assertEqual('deep-thought-autolanding',
                         str(self.jenkins_job))


class TestCoverageReporting(TestCase):
    fixtures = ['phablet-apps.json']

    def setUp(self):
        self.jenkins_job = JenkinsJob.objects.get(
            name='phone-app-quantal-i386-autolanding')

    # TODO Fix this test, fixture needs aggregated coverage report
    #def test_coverage_exists(self):
    #    self.assertIsNone(
    #        self.jenkins_job.coverage_report)

    # TODO Fix this test, fixture needs aggregated coverage report
    #def test_line_coverage_is_correct(self):
    #    self.assertAlmostEqual(0.44,
    #                           self.jenkins_job.line_coverage,
    #                           1)

    # TODO Fix this test, fixture needs aggregated coverage report
    #def test_branch_coverage_is_correct(self):
    #    self.assertAlmostEqual(0.53,
    #                           self.jenkins_job.branch_coverage,
    #                           1)


class TestCoverageReportingOnPrimaryJob(TestCase):
    """Some jobs aren't equipped with coverage--just return None."""

    fixtures = ['phablet-apps.json']

    def setUp(self):
        self.jenkins_job = JenkinsJob.objects.get(
            name='phone-app-autolanding')

    # TODO Fix this test, fixture needs aggregated coverage report
    #def test_coverage_doesnt_exist(self):
    #    self.assertIsNotNone(
    #        self.jenkins_job.coverage_report)

    # TODO Fix this test, fixture needs aggregated coverage report
    #def test_line_coverage_is_correct(self):
    #    self.assertAlmostEqual(0.44,
    #                           self.jenkins_job.line_coverage,
    #                           1)

    # TODO Fix this test, fixture needs aggregated coverage report
    #def test_branch_coverage_is_correct(self):
    #    self.assertAlmostEqual(0.53,
    #                           self.jenkins_job.branch_coverage,
    #                           1)

    # TODO Fix this test, fixture needs aggregated coverage report
    #def test_line_count_is_correct(self):
    #    self.assertEqual(???,
    #                           self.jenkins_job.branch_coverage,
    #                           1)

    # TODO Fix this test, fixture needs aggregated coverage report
    #def test_hit_count_is_correct(self):
    #    self.assertEqual(???,
    #                           self.jenkins_job.branch_coverage,
    #                           1)

    # TODO Fix this test, fixture needs aggregated coverage report
    #def test_taken_count_is_correct(self):
    #    self.assertEqual(???,
    #                           self.jenkins_job.branch_coverage,
    #                           1)

    # TODO Fix this test, fixture needs aggregated coverage report
    #def test_total_count_is_correct(self):
    #    self.assertEqual(???,
    #                           self.jenkins_job.branch_coverage,
    #                           1)


class TestNoCoverageReporting(TestCase):
    """Some jobs aren't equipped with coverage--just return None."""

    fixtures = ['phablet-apps.json']

    def setUp(self):
        self.jenkins_job = JenkinsJob.objects.get(
            name='camera-app-autolanding')

    def test_line_coverage_is_correct(self):
        self.assertIsNone(self.jenkins_job.line_coverage)

    def test_branch_coverage_is_correct(self):
        self.assertIsNone(self.jenkins_job.branch_coverage)


class TestLastFailCount(TestCase):
    fixtures = ['phablet-apps.json']

    # TODO Fix with update fixture
    #def test_last_fail_count_is_correct(self):
    #    self.assertEqual(0, self.jenkins_job.last_fail_count)


class TestLastPassCount(TestCase):
    fixtures = ['phablet-apps.json']

    # TODO Fix with update fixture
    #def test_last_pass_count_is_correct(self):
    #    self.assertEqual(0, self.jenkins_job.last_pass_count)


class TestLastSkipCount(TestCase):
    fixtures = ['phablet-apps.json']

    # TODO Fix with update fixture
    #def test_last_skip_count_is_correct(self):
    #    self.assertEqual(0, self.jenkins_job.last_skip_count)


class TestLastDuration(TestCase):
    fixtures = ['phablet-apps.json']

    # TODO Fix with update fixture
    #def test_last_duration_is_correct(self):
    #    self.assertEqual(0, self.jenkins_job.last_duration)
