# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import call, MagicMock, patch
import json
import os.path
import unittest
from ci.util.extractor import get_json_from_url, get_text_from_url

"""Tests for module JenkinsUtilites"""

DATA_PATH = "tests/data"


class JenkinsUtilitiesTestCase(unittest.TestCase):
    """Unit tests for JenkinsUtilities methods"""

    def setUp(self):
        self.get_mock = MagicMock(return_value=type('fake_request',
                                                    (object,),
                                                    {'status_code': 200,
                                                     'text': '{"value": 42}'}))

    def test_get_json_from_url(self):
        url = "http://localhost/foo/"
        data = None
        with patch('requests.get', self.get_mock):
            data = get_json_from_url(url)
        self.assertEqual(data['value'], 42)
        self.assertEqual(self.get_mock.call_args, call(url + "/api/json"))

    def test_get_json_from_url_none(self):
        url = "http://localhost/foo/"
        mock = MagicMock(return_value=type('fake_request',
                                           (object,),
                                           {'status_code': 404,
                                            'text': 'null'}))
        with patch('requests.get', mock):
            with self.assertRaises(EnvironmentError) as err:
                get_json_from_url(url)
            self.assertEqual("No json data at http://localhost/foo//api/json",
                             "%s" % (err.exception))

    def test_get_text_from_url(self):
        url = "http://localhost/foo/"
        data = None
        with patch('requests.get', self.get_mock):
            data = get_text_from_url(url)
        self.assertEqual(data, '{"value": 42}')
        self.assertEqual(self.get_mock.call_args, call(url))

    def test_get_text_from_url_none(self):
        url = "http://localhost/foo/"
        mock = MagicMock(return_value=type('fake_request',
                                           (object,),
                                           {'status_code': 404,
                                            'text': 'null'}))
        with patch('requests.get', mock):
            with self.assertRaises(EnvironmentError) as err:
                get_text_from_url(url)
            self.assertEqual("No data at http://localhost/foo/",
                             "%s" % (err.exception))
