# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from unittest import skip
from mock import MagicMock, patch, call
from django.test import TestCase
from ci.models import LaunchpadProject
import ci.util.launchpad_project as util_launchpad_project


class TestCreateFromName(TestCase):

    def setUp(self):
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()

    def tearDown(self):
        self.logger_patch.stop()

    # TODO restore this when we're really dealing with Launchpad
    @skip
    def test_vanilla(self):
        dummy_launchpad_project_name = 'fake-ci-project'
        dummy_launchpad_project_url = 'http://this.is.an.official.lp.url/really/'
        # TODO this could be miraculously clearer :)
        lp = MagicMock(
            return_value=MagicMock(
                projects={dummy_launchpad_project_name:
                              MagicMock(web_link=dummy_launchpad_project_url)}))
        with patch('launchpadlib.launchpad.Launchpad.login_with', lp), \
            patch('ci.models.LaunchpadProject.objects.create') as launchpad_project_create:
            result = util_launchpad_project.create_launchpad_project_from_name(
                name=dummy_launchpad_project_name)
        launchpad_project_create.assert_called_with(
            name=dummy_launchpad_project_name,
            url=dummy_launchpad_project_url)
        # TODO need to actually check for the url attribute

    def test_naming(self):
        launchpad_project_get_or_create = MagicMock(
            return_value = ("foo", False))
        with patch('ci.models.LaunchpadProject.objects.get_or_create',
                   launchpad_project_get_or_create):
            result = util_launchpad_project.create_launchpad_project_from_name(
                "foo")
        launchpad_project_get_or_create.assert_called()
