# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ci.models import CiProject, LaunchpadProject, Cu2dStack, Release
from ci.util.jenkins_job import create_jenkins_job_from_name
from ci.util.launchpad_project import create_launchpad_project_from_name


def get_launchpad_name_from_project_name(project_name):
    """Launchpad name is the last term of a triplet."""

    return project_name.split('.')[-1]


def create_ci_project(config):
    launchpad_name = get_launchpad_name_from_project_name(config['name'])
    try:
        launchpad_project = LaunchpadProject.objects.get(
            name=launchpad_name,
        )
    except LaunchpadProject.DoesNotExist:
        launchpad_project = create_launchpad_project_from_name(
            name=launchpad_name,
        )
    stack = Cu2dStack.objects.get(name=config['stack'])
    release = Release.objects.get(name=config['release'])
    jenkins_server = config['server']

    ci_job = create_jenkins_job_from_name(
        jenkins_server,
        config['ci'],
    )
    al_job = create_jenkins_job_from_name(
        jenkins_server,
        config['autolanding'],
    )

    ci_project, new_ci_project = CiProject.objects.get_or_create(
        name=config['name'],
        launchpad_project=launchpad_project,
        stack=stack,
        release=release,
        ci_primary_job=ci_job,
        autolanding_primary_job=al_job,
    )

    if ci_job is not None and new_ci_project:
        ci_job.project = ci_project
        ci_job.save()

    if al_job is not None and new_ci_project:
        al_job.project = ci_project
        al_job.save()
