# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
from django.db import IntegrityError, transaction
from ci.models import (JenkinsFlavor,
                         JenkinsRun,
                         Series,
                         JenkinsResult)
import ci.util.extractor as extractor

logger = logging.getLogger("qa_dashboard")


def create_jenkins_run_from_url(url, jenkins_build):
    extractor_run = extractor.JenkinsRun(url)
    flavor, created = JenkinsFlavor.objects.get_or_create(name=extractor_run.flavor)
    series, created = Series.objects.get_or_create(name=extractor_run.series)
    try:
        result, created = JenkinsResult.objects.get_or_create(name=extractor_run.result)
    except IntegrityError, e:
        # no builds yet, IntegrityError on None name
        transaction.rollback()
        return None
    try:
        logger.debug("creating JenkinsRun at {}".format(url))
        jenkins_run = JenkinsRun.objects.create(
            jenkins_build=jenkins_build,
            number=extractor_run.number,
            name=extractor_run.name,
            url=url,
            duration=extractor_run.duration.seconds,
            timestamp=extractor_run.timestamp,
            flavor=flavor,
            series=series,
            result=result)
    except IntegrityError:
        transaction.rollback()
        logger.debug("Duplicate Jenkins run found at {}.".format(url))
        # TODO make it possible to update a run
        return None
    except TypeError, e:
        logger.debug("Error creating JenkinsRun: {}".format(e))
        return None
    # Disabling this until we have a JenkinsRun-specific JenkinsReport create
    #logger.debug("Importing coverage reports associated with JenkinsRun at {}.".format(url))
    # TODO FJG Fix this
    #try:
    #    jenkins_run.coverage_report = JenkinsCoverageReport.objects.create(
    #        # FIXME: restore
    #        # run = jenkins_run,
    #        xml_text=extractor_run.coverage_report.xml_text,
    #        line_rate=extractor_run.coverage_report.line_rate,
    #        branch_rate=extractor_run.coverage_report.branch_rate)
    #    jenkins_run.save()
    #except EnvironmentError:
    #    # no data was found at the coverage report url
    #    pass
    #logger.debug("importing test reports associated with JenkinsRun at {}".format(url))
    #try:
    #    jenkins_run.test_report = JenkinsTestReport.objects.create(
    #        fail_count=extractor_run.test_report.fail_count,
    #        pass_count=extractor_run.test_report.pass_count,
    #        skip_count=extractor_run.test_report.skip_count,
    #        duration=extractor_run.test_report.duration.seconds)
    #    jenkins_run.save()
    #except EnvironmentError:
    #    # no data was found at the test report url
    #    pass
    #except EnvironmentError, e:
    #    logger.error(e)
    return jenkins_run
