# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
from django.template import RequestContext
from django.shortcuts import get_object_or_404, render_to_response

from ci.models import (
    Cu2dStack,
    Release,
    CiProject,
    JenkinsJob,
    JenkinsBuild,
    JenkinsResult,
    LaunchpadProject,
)

logger = logging.getLogger('qa_dashboard')


BRANCH_FILTER_TYPES = ['all',
                       'autolanding',
                       'ci',
                       'daily',
                       'coverity', ]


def job(request, job_name):
    result_filter = request.GET.get('show')
    job = JenkinsJob.objects.get(name=job_name)
    result_list = JenkinsResult.objects.all()
    if result_filter is None or result_filter not in [
            result.name for result in result_list]:
        result_filter = 'all'
    if result_filter == 'all':
        build_list = JenkinsBuild.objects.filter(
            jenkins_job=job)
    else:
        build_list = JenkinsBuild.objects.filter(
            jenkins_job=job,
            result__name=result_filter)
    data = {
        'job': job,
        'build_list': build_list,
        'result_list': result_list,
    }
    return render_to_response(
        'ci/job.html',
        data,
        RequestContext(request)
    )


def stack_list(request):
    show_filter = request.GET.get('show')
    release_list = Release.objects.all()
    if show_filter:
        show_release = Release.objects.filter(name=show_filter)
        stack_list = Cu2dStack.objects.filter(release=show_release)
        project_list = CiProject.objects.filter(
            stack__name__startswith=show_filter)
    else:
        stack_list = Cu2dStack.objects.all()
        project_list = CiProject.objects.all()
        show_release = None
        show_filter = None
    data = {
        'stack_list': stack_list,
        'release_list': release_list,
        'project_list': project_list,
        'show_filter': show_filter,
    }
    return render_to_response(
        'ci/daily_release_overview.html',
        data,
        RequestContext(request)
    )


def project(request, name):
    project = get_object_or_404(
        CiProject,
        launchpad_project__name=name,
        release__name='head')
    try:
        stack = Cu2dStack.objects.get(name=project.stack.name)
    except AttributeError:
        # it's possible that the stack may not be fully created
        stack = None
    job_list = JenkinsJob.objects.filter(project=project)

    coverage_builds = JenkinsBuild.objects.filter(
        jenkins_job=project.autolanding_primary_job,
        coverage_report__isnull=False).order_by('timestamp')
    data = []
    for build in coverage_builds:
        data.append({'timestamp': build.timestamp,
                     'line coverage': build.line_coverage,
                     'branch coverage': build.branch_coverage})
    data = {
        'stack': stack,
        'project': project,
        'job_list': job_list,
    }
    return render_to_response(
        'ci/project_detail.html',
        data,
        RequestContext(request)
    )


def project_list(request):
    release_filter = request.GET.get('show')
    if release_filter:
        project_list = LaunchpadProject.objects.filter(
            ciproject__release__name=release_filter)
    else:
        project_list = LaunchpadProject.objects.all()
    release_list = Release.objects.all()
    data = {
        'project_list': project_list,
        'release_list': release_list,
    }
    return render_to_response(
        'ci/project_list.html',
        data,
        RequestContext(request)
    )
